from south.modelsinspector import add_introspection_rules


add_introspection_rules([], ['^mutant\.db\.fields\.python\.PythonIdentifierField'])
add_introspection_rules([], ['^mutant\.db\.fields\.python\.RegExpStringField'])
add_introspection_rules([], ['^mutant\.db\.fields\.python\.DirectoryPathField'])
add_introspection_rules([], ['^mutant\.db\.fields\.translation\.LazilyTranslatedField'])
